/*-
 * See the file LICENSE for redistribution information.
 *
 * Copyright (c) 1997-2002
 *	Sleepycat Software.  All rights reserved.
 */

#include "db_config.h"

#ifndef lint
static const char revid[] = "$Id: os_seek.c,v 1.1.1.1 2004/08/19 23:53:56 gopalan Exp $";
#endif /* not lint */

#ifndef __KERNEL__

#ifndef NO_SYSTEM_INCLUDES
#include <sys/types.h>

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#endif

#else

#include <linux/types.h>
#include <linux/string.h>

#endif
#include "db_int.h"

/*
 * __os_seek --
 *	Seek to a page/byte offset in the file.
 *
 * PUBLIC: int __os_seek __P((DB_ENV *,
 * PUBLIC:      DB_FH *, size_t, db_pgno_t, u_int32_t, int, DB_OS_SEEK));
 */
int
__os_seek(dbenv, fhp, pgsize, pageno, relative, isrewind, db_whence)
	DB_ENV *dbenv;
	DB_FH *fhp;
	size_t pgsize;
	db_pgno_t pageno;
	u_int32_t relative;
	int isrewind;
	DB_OS_SEEK db_whence;
{
	off_t offset;
	int ret, whence;
	
	print_entry_location();

	switch (db_whence) {
	case DB_OS_SEEK_CUR:
#ifndef __KERNEL__
		whence = SEEK_CUR;
#else
		whence = 1;
#endif /* __KERNEL__ */
		break;
	case DB_OS_SEEK_END:
#ifndef __KERNEL__
		whence = SEEK_END; 
#else
		whence = 2;
#endif /* __KERNEL__ */
		break;
	case DB_OS_SEEK_SET:
#ifndef __KERNEL__
		whence = SEEK_SET; 
#else
		whence = 0;
#endif /* __KERNEL__ */
		break;
	default:
		print_exit_location(EINVAL);
		return (EINVAL);
	}

	if (DB_GLOBAL(j_seek) != NULL)
		ret = DB_GLOBAL(j_seek)(fhp->fd,
		    pgsize, pageno, relative, isrewind, whence);
	else {
		offset = (off_t)pgsize * pageno + relative;
		if (isrewind)
			offset = -offset;
		do {
#ifndef __KERNEL__
			ret = lseek(fhp->fd, offset, whence) == -1 ? __os_get_errno() : 0;
#else
			ret = generic_file_llseek(fhp->fd, offset, whence) == -1 ? __os_get_errno() : 0;
#endif
		} while (ret == EINTR);
	}

	if (ret == 0) {
		fhp->pgsize = pgsize;
		fhp->pgno = pageno;
		fhp->offset = relative;
	} 
#ifndef __KERNEL__
	else
		__db_err(dbenv, "seek: %lu %d %d: %s",
		    (u_long)pgsize * pageno + relative,
		    isrewind, db_whence, strerror(ret));
#endif

	print_exit_location(ret);
	return (ret);
}
